{
  FreePascal translation of libhal.h
}
(***************************************************************************
 * CVSID: $Id$
 *
 * libhal.h : HAL daemon C convenience library headers
 *
 * Copyright (C) 2003 David Zeuthen, <david@fubar.dk>
 *
 * Licensed under the Academic Free License version 2.1
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *
 **************************************************************************)

unit libhal;

{$IFDEF FPC}
{$MODE objfpc}{$H+}
{$PACKRECORDS C}
{$ENDIF}

interface

uses
  dbus;

const hallib = 'libhal';

type
  PLibHalContext = Pointer;
  PLibHalChangeSet = Pointer;
  PLibHalPropertySet = Pointer;
  PLibHalProperty = Pointer;

  (**
   * LibHalPropertyType:
   *
   * Possible types for properties on hal device objects
   *)

  PLibHalPropertyType = ^LibHalPropertyType;
  LibHalPropertyType = (
    //Used to report error condition
    LIBHAL_PROPERTY_TYPE_INVALID := DBUS_TYPE_INVALID,
    //Type for 32-bit signed integer property
    LIBHAL_PROPERTY_TYPE_INT32 := DBUS_TYPE_INT32,
    //Type for 64-bit unsigned integer property
    LIBHAL_PROPERTY_TYPE_UINT64 := DBUS_TYPE_UINT64,
    //Type for double precision floating point property
    LIBHAL_PROPERTY_TYPE_DOUBLE := DBUS_TYPE_DOUBLE,
    //Type for boolean property
    LIBHAL_PROPERTY_TYPE_BOOLEAN := DBUS_TYPE_BOOLEAN,
    //Type for UTF-8 string property
    LIBHAL_PROPERTY_TYPE_STRING := DBUS_TYPE_STRING,
    //Type for list of UTF-8 strings property
    LIBHAL_PROPERTY_TYPE_STRLIST := (longint(DBUS_TYPE_STRING shl 8)) + integer('l')
    );

  PLibHalPropertySetIterator = ^LibHalPropertySetIterator;
  LibHalPropertySetIterator = record
    pset: PLibHalPropertySet;
    idx: dword;
    cur_prop: PLibHalProperty;
    reservered0: pointer;
    reservered1: pointer;
  end;

  {*
   * LibHalIntegrateDBusIntoMainLoop:
   * @ctx: context for connection to hald
   * @dbus_connection: DBus connection to use in ctx
   *
   * Type for function in application code that integrates a
   * DBusConnection object into its own mainloop.
  }
  LibHalIntegrateDBusIntoMainLoop = procedure(ctx: pLibHalContext; dbus_connection: pDBusConnection); cdecl;

  {*
   * LibHalDeviceAdded:
   * @ctx: context for connection to hald
   * @udi: the Unique Device Id
   *
   * Type for callback when a device is added.
    }
  LibHalDeviceAdded = procedure(ctx: pLibHalContext; udi: pchar); cdecl;

  {*
   * LibHalDeviceRemoved:
   * @ctx: context for connection to hald
   * @udi: the Unique Device Id
   *
   * Type for callback when a device is removed.
    }
  LibHalDeviceRemoved = procedure(ctx: pLibHalContext; udi: pchar); cdecl;
  {*
   * LibHalDeviceNewCapability:
   * @ctx: context for connection to hald
   * @udi: the Unique Device Id
   * @capability: capability of the device
   *
   * Type for callback when a device gains a new capability.
   *
    }
  LibHalDeviceNewCapability = procedure(ctx: pLibHalContext; udi: pchar; capability: pchar); cdecl;

  {*
   * LibHalDeviceLostCapability:
   * @ctx: context for connection to hald
   * @udi: the Unique Device Id
   * @capability: capability of the device
   *
   * Type for callback when a device loses a capability.
   *
    }
  LibHalDeviceLostCapability = procedure(ctx: pLibHalContext; udi: pchar; capability: pchar); cdecl;

  {*
   * LibHalDevicePropertyModified:
   * @ctx: context for connection to hald
   * @udi: the Unique Device Id
   * @key: name of the property that has changed
   * @is_removed: whether or not property was removed
   * @is_added: whether or not property was added
   *
   * Type for callback when a property of a device changes.
    }
  LibHalDevicePropertyModified = procedure(ctx: pLibHalContext; udi: pchar; key: pchar; is_removed: dbus_bool_t; is_added: dbus_bool_t); cdecl;

  {*
   * LibHalDeviceCondition:
   * @ctx: context for connection to hald
   * @udi: the Unique Device Id
   * @condition_name: name of the condition, e.g. ProcessorOverheating. Consult the HAL spec for details
   * @condition_detail: detail of condition
   *
   * Type for callback when a non-continuous condition occurs on a device.
    }
  LibHalDeviceCondition = procedure(ctx: pLibHalContext; udi: pchar; condition_name: pchar; condition_detail: pchar); cdecl;

  {*
   * LibHalGlobalInterfaceLockAcquired:
   * @ctx: context for connection to hald
   * @interface_name: the name of the interface
   * @lock_owner: what service acquired the lock
   * @num_locks: number of locks on the interface
   *
   * Type for callback when someone acquires a global lock.
    }
  LibHalGlobalInterfaceLockAcquired = procedure(ctx: pLibHalContext; interface_name: pchar; lock_owner: pchar; num_locks: longint); cdecl;

  {*
   * LibHalGlobalInterfaceLockReleased:
   * @ctx: context for connection to hald
   * @interface_name: the name of the interface
   * @lock_owner: what service released the lock
   * @num_locks: number of locks on the interface
   *
   * Type for callback when someone releases a global lock.
    }
  LibHalGlobalInterfaceLockReleased = procedure(ctx: pLibHalContext; interface_name: pchar; lock_owner: pchar; num_locks: longint); cdecl;

  {*
   * LibHalInterfaceLockAcquired:
   * @ctx: context for connection to hald
   * @udi: the Unique Device Id
   * @interface_name: the name of the interface
   * @lock_owner: what service acquired the lock
   * @num_locks: number of locks on the interface
   *
   * Type for callback when someone acquires a lock on a device.
    }

  LibHalInterfaceLockAcquired = procedure(ctx: pLibHalContext; udi: pchar; interface_name: pchar; lock_owner: pchar; num_locks: longint); cdecl;
  {*
   * LibHalInterfaceLockReleased:
   * @ctx: context for connection to hald
   * @udi: the Unique Device Id
   * @interface_name: the name of the interface
   * @lock_owner: what service released the lock
   * @num_locks: number of locks on the interface
   *
   * Type for callback when someone acquires a lock on a device.
    }
  LibHalInterfaceLockReleased = procedure(ctx: pLibHalContext; udi: pchar; interface_name: pchar; lock_owner: pchar; num_locks: longint); cdecl;

  { Create a new context for a connection with hald  }
function libhal_ctx_new: PLibHalContext; cdecl; external hallib name 'libhal_ctx_new';

{ Enable or disable caching  }
function libhal_ctx_set_cache(ctx: pLibHalContext; use_cache: dbus_bool_t): dbus_bool_t; cdecl; external hallib name 'libhal_ctx_set_cache';

{ Set DBus connection to use to talk to hald.  }
function libhal_ctx_set_dbus_connection(ctx: pLibHalContext; conn: pDBusConnection): dbus_bool_t; cdecl; external hallib name 'libhal_ctx_set_dbus_connection';

{ Get DBus connection to use to talk to hald.  }
function libhal_ctx_get_dbus_connection(ctx: pLibHalContext): PDBusConnection; cdecl; external hallib name 'libhal_ctx_get_dbus_connection';

{ Set user data for the context  }
function libhal_ctx_set_user_data(ctx: pLibHalContext; user_data: pointer): dbus_bool_t; cdecl; external hallib name 'libhal_ctx_set_user_data';

{ Get user data for the context  }
function libhal_ctx_get_user_data(ctx: pLibHalContext): pointer; cdecl; external hallib name 'libhal_ctx_get_user_data';

{ Set the callback for when a device is added  }
function libhal_ctx_set_device_added(ctx: pLibHalContext; callback: LibHalDeviceAdded): dbus_bool_t; cdecl; external hallib name 'libhal_ctx_set_device_added';

{ Set the callback for when a device is removed  }
function libhal_ctx_set_device_removed(ctx: pLibHalContext; callback: LibHalDeviceRemoved): dbus_bool_t; cdecl; external hallib name 'libhal_ctx_set_device_removed';

{ Set the callback for when a device gains a new capability  }
function libhal_ctx_set_device_new_capability(ctx: pLibHalContext; callback: LibHalDeviceNewCapability): dbus_bool_t; cdecl; external hallib name 'libhal_ctx_set_device_new_capability';

{ Set the callback for when a device loses a capability  }
function libhal_ctx_set_device_lost_capability(ctx: pLibHalContext; callback: LibHalDeviceLostCapability): dbus_bool_t; cdecl; external hallib name 'libhal_ctx_set_device_lost_capability';

{ Set the callback for when a property is modified on a device  }
function libhal_ctx_set_device_property_modified(ctx: pLibHalContext; callback: LibHalDevicePropertyModified): dbus_bool_t; cdecl; external hallib name 'libhal_ctx_set_device_property_modified';

{ Set the callback for when a device emits a condition  }
function libhal_ctx_set_device_condition(ctx: pLibHalContext; callback: LibHalDeviceCondition): dbus_bool_t; cdecl; external hallib name 'libhal_ctx_set_device_condition';

{ Set the callback for when a global interface lock is acquired   }
function libhal_ctx_set_global_interface_lock_acquired(ctx: pLibHalContext; callback: LibHalGlobalInterfaceLockAcquired): dbus_bool_t; cdecl; external hallib name 'libhal_ctx_set_global_interface_lock_acquired';

{ Set the callback for when a global interface lock is released   }
function libhal_ctx_set_global_interface_lock_released(ctx: pLibHalContext; callback: LibHalGlobalInterfaceLockReleased): dbus_bool_t; cdecl; external hallib name 'libhal_ctx_set_global_interface_lock_released';

{ Set the callback for when an interface lock is acquired   }
function libhal_ctx_set_interface_lock_acquired(ctx: pLibHalContext; callback: LibHalInterfaceLockAcquired): dbus_bool_t; cdecl; external hallib name 'libhal_ctx_set_interface_lock_acquired';

{ Set the callback for when an interface lock is released   }
function libhal_ctx_set_interface_lock_released(ctx: pLibHalContext; callback: LibHalInterfaceLockReleased): dbus_bool_t; cdecl; external hallib name 'libhal_ctx_set_interface_lock_released';

{ Initialize the connection to hald  }
function libhal_ctx_init(ctx: pLibHalContext; error: pDBusError): dbus_bool_t; cdecl; external hallib name 'libhal_ctx_init';

{ Shut down a connection to hald  }
function libhal_ctx_shutdown(ctx: pLibHalContext; error: pDBusError): dbus_bool_t; cdecl; external hallib name 'libhal_ctx_shutdown';

{ Free a LibHalContext resource  }
function libhal_ctx_free(ctx: pLibHalContext): dbus_bool_t; cdecl; external hallib name 'libhal_ctx_free';

{ Create an already initialized connection to hald  }
function libhal_ctx_init_direct(error: pDBusError): PLibHalContext; cdecl; external hallib name 'libhal_ctx_init_direct';

{ Get all devices in the Global Device List (GDL).  }
function libhal_get_all_devices(ctx: pLibHalContext; num_devices: plongint; error: pDBusError): PPchar; cdecl; external hallib name 'libhal_get_all_devices';

{ Determine if a device exists.  }
function libhal_device_exists(ctx: pLibHalContext; udi: pchar; error: pDBusError): dbus_bool_t; cdecl; external hallib name 'libhal_device_exists';

{ Print a device to stdout; useful for debugging.  }
function libhal_device_print(ctx: pLibHalContext; udi: pchar; error: pDBusError): dbus_bool_t; cdecl; external hallib name 'libhal_device_print';

{ Determine if a property on a device exists.  }
function libhal_device_property_exists(ctx: pLibHalContext; udi: pchar; key: pchar; error: pDBusError): dbus_bool_t; cdecl; external hallib name 'libhal_device_property_exists';

{ Get the value of a property of type string.  }
function libhal_device_get_property_string(ctx: pLibHalContext; udi: pchar; key: pchar; error: pDBusError): Pchar; cdecl; external hallib name 'libhal_device_get_property_string';

{ Get the value of a property of type signed integer.  }
function libhal_device_get_property_int(ctx: pLibHalContext; udi: pchar; key: pchar; error: pDBusError): dbus_int32_t; cdecl; external hallib name 'libhal_device_get_property_int';

{ Get the value of a property of type unsigned integer.  }
function libhal_device_get_property_uint64(ctx: pLibHalContext; udi: pchar; key: pchar; error: pDBusError): dbus_uint64_t; cdecl; external hallib name 'libhal_device_get_property_uint64';

{ Get the value of a property of type double.  }
function libhal_device_get_property_double(ctx: pLibHalContext; udi: pchar; key: pchar; error: pDBusError): double; cdecl; external hallib name 'libhal_device_get_property_double';

{ Get the value of a property of type bool.  }
function libhal_device_get_property_bool(ctx: pLibHalContext; udi: pchar; key: pchar; error: pDBusError): dbus_bool_t; cdecl; external hallib name 'libhal_device_get_property_bool';

{ Get the value of a property of type string list.  }
function libhal_device_get_property_strlist(ctx: pLibHalContext; udi: pchar; key: pchar; error: pDBusError): PPchar; cdecl; external hallib name 'libhal_device_get_property_strlist';

{ Set a property of type string.  }
function libhal_device_set_property_string(ctx: pLibHalContext; udi: pchar; key: pchar; value: pchar; error: pDBusError): dbus_bool_t; cdecl; external hallib name 'libhal_device_set_property_string';

{ Set a property of type signed integer.  }
function libhal_device_set_property_int(ctx: pLibHalContext; udi: pchar; key: pchar; value: dbus_int32_t; error: pDBusError): dbus_bool_t; cdecl; external hallib name 'libhal_device_set_property_int';

{ Set a property of type unsigned integer.  }
function libhal_device_set_property_uint64(ctx: pLibHalContext; udi: pchar; key: pchar; value: dbus_uint64_t; error: pDBusError): dbus_bool_t; cdecl; external hallib name 'libhal_device_set_property_uint64';

{ Set a property of type double.  }
function libhal_device_set_property_double(ctx: pLibHalContext; udi: pchar; key: pchar; value: double; error: pDBusError): dbus_bool_t; cdecl; external hallib name 'libhal_device_set_property_double';

{ Set a property of type bool.  }
function libhal_device_set_property_bool(ctx: pLibHalContext; udi: pchar; key: pchar; value: dbus_bool_t; error: pDBusError): dbus_bool_t; cdecl; external hallib name 'libhal_device_set_property_bool';

{ Append to a property of type strlist.  }
function libhal_device_property_strlist_append(ctx: pLibHalContext; udi: pchar; key: pchar; value: pchar; error: pDBusError): dbus_bool_t; cdecl; external hallib name 'libhal_device_property_strlist_append';

{ Prepend to a property of type strlist.  }
function libhal_device_property_strlist_prepend(ctx: pLibHalContext; udi: pchar; key: pchar; value: pchar; error: pDBusError): dbus_bool_t; cdecl; external hallib name 'libhal_device_property_strlist_prepend';

{ Remove a specified string from a property of type strlist.  }
function libhal_device_property_strlist_remove_index(ctx: pLibHalContext; udi: pchar; key: pchar; idx: dword; error: pDBusError): dbus_bool_t; cdecl; external hallib name 'libhal_device_property_strlist_remove_index';

{ Remove a specified string from a property of type strlist.  }
function libhal_device_property_strlist_remove(ctx: pLibHalContext; udi: pchar; key: pchar; value: pchar; error: pDBusError): dbus_bool_t; cdecl; external hallib name 'libhal_device_property_strlist_remove';

{ Remove a property.  }
function libhal_device_remove_property(ctx: pLibHalContext; udi: pchar; key: pchar; error: pDBusError): dbus_bool_t; cdecl; external hallib name 'libhal_device_remove_property';

{ Query a property type of a device.  }
function libhal_device_get_property_type(ctx: pLibHalContext; udi: pchar; key: pchar; error: pDBusError): LibHalPropertyType; cdecl; external hallib name 'libhal_device_get_property_type';

function libhal_device_new_changeset(udi: pchar): pLibHalChangeSet; cdecl; external hallib name 'libhal_device_new_changeset';
function libhal_changeset_set_property_string(changeset: pLibHalChangeSet; key: pchar; value: pchar): dbus_bool_t; cdecl; external hallib name 'libhal_changeset_set_property_string';
function libhal_changeset_set_property_int(changeset: pLibHalChangeSet; key: pchar; value: dbus_int32_t): dbus_bool_t; cdecl; external hallib name 'libhal_changeset_set_property_int';
function libhal_changeset_set_property_uint64(changeset: pLibHalChangeSet; key: pchar; value: dbus_uint64_t): dbus_bool_t; cdecl; external hallib name 'libhal_changeset_set_property_uint64';
function libhal_changeset_set_property_double(changeset: pLibHalChangeSet; key: pchar; value: double): dbus_bool_t; cdecl; external hallib name 'libhal_changeset_set_property_double';
function libhal_changeset_set_property_bool(changeset: pLibHalChangeSet; key: pchar; value: dbus_bool_t): dbus_bool_t; cdecl; external hallib name 'libhal_changeset_set_property_bool';
function libhal_changeset_set_property_strlist(changeset: pLibHalChangeSet; key: pchar; value: Ppchar): dbus_bool_t; cdecl; external hallib name 'libhal_changeset_set_property_strlist';
function libhal_device_commit_changeset(ctx: pLibHalContext; changeset: pLibHalChangeSet; error: pDBusError): dbus_bool_t; cdecl; external hallib name 'libhal_device_commit_changeset';
procedure libhal_device_free_changeset(changeset: pLibHalChangeSet); cdecl; external hallib name 'libhal_device_free_changeset';

function libhal_device_get_all_properties(ctx: pLibHalContext; udi: pchar; error: pDBusError): PLibHalPropertySet; cdecl; external hallib name 'libhal_device_get_all_properties';

{ sort all properties according to property name  }
procedure libhal_property_set_sort(pset: pLibHalPropertySet); cdecl; external hallib name 'libhal_property_set_sort';

{ Free a property pset earlier obtained with libhal_device_get_all_properties().  }
procedure libhal_free_property_set(pset: pLibHalPropertySet); cdecl; external hallib name 'libhal_free_property_set';

{ Get the number of properties in a property pset.  }
function libhal_property_set_get_num_elems(pset: pLibHalPropertySet): dword; cdecl; external hallib name 'libhal_property_set_get_num_elems';

{ Get type of property.  }
function libhal_ps_get_type(pset: pLibHalPropertySet; key: pchar): LibHalPropertyType; cdecl; external hallib name 'libhal_ps_get_type';

{ Get the value of a property of type string.  }
function libhal_ps_get_string(pset: pLibHalPropertySet; key: pchar): Pchar; cdecl; external hallib name 'libhal_ps_get_string';

{ Get the value of a property of type signed integer.  }
function libhal_ps_get_int32(pset: pLibHalPropertySet; key: pchar): dbus_int32_t; cdecl; external hallib name 'libhal_ps_get_int32';

{ Get the value of a property of type unsigned integer.  }
function libhal_ps_get_uint64(pset: pLibHalPropertySet; key: pchar): dbus_uint64_t; cdecl; external hallib name 'libhal_ps_get_uint64';

{ Get the value of a property of type double.  }
function libhal_ps_get_double(pset: pLibHalPropertySet; key: pchar): double; cdecl; external hallib name 'libhal_ps_get_double';

{ Get the value of a property of type bool.  }
function libhal_ps_get_bool(pset: pLibHalPropertySet; key: pchar): dbus_bool_t; cdecl; external hallib name 'libhal_ps_get_bool';

{ Get the value of a property of type string list.  }
function libhal_ps_get_strlist(pset: pLibHalPropertySet; key: pchar): PPchar; cdecl; external hallib name 'libhal_ps_get_strlist';

{ Initialize a property set iterator.  }
procedure libhal_psi_init(iter: pLibHalPropertySetIterator; pset: pLibHalPropertySet); cdecl; external hallib name 'libhal_psi_init';

{ Determine whether there are more properties to iterate over  }
function libhal_psi_has_more(iter: pLibHalPropertySetIterator): dbus_bool_t; cdecl; external hallib name 'libhal_psi_has_more';

{ Advance iterator to next property.  }
procedure libhal_psi_next(iter: pLibHalPropertySetIterator); cdecl; external hallib name 'libhal_psi_next';

{ Get type of property.  }
function libhal_psi_get_type(iter: pLibHalPropertySetIterator): LibHalPropertyType; cdecl; external hallib name 'libhal_psi_get_type';

{ Get the key of a property.  }
function libhal_psi_get_key(iter: pLibHalPropertySetIterator): Pchar; cdecl; external hallib name 'libhal_psi_get_key';

{ Get the value of a property of type string.  }
function libhal_psi_get_string(iter: pLibHalPropertySetIterator): Pchar; cdecl; external hallib name 'libhal_psi_get_string';

{ Get the value of a property of type signed integer.  }
function libhal_psi_get_int(iter: pLibHalPropertySetIterator): dbus_int32_t; cdecl; external hallib name 'libhal_psi_get_int';

{ Get the value of a property of type unsigned integer.  }
function libhal_psi_get_uint64(iter: pLibHalPropertySetIterator): dbus_uint64_t; cdecl; external hallib name 'libhal_psi_get_uint64';

{ Get the value of a property of type double.  }
function libhal_psi_get_double(iter: pLibHalPropertySetIterator): double; cdecl; external hallib name 'libhal_psi_get_double';

{ Get the value of a property of type bool.  }
function libhal_psi_get_bool(iter: pLibHalPropertySetIterator): dbus_bool_t; cdecl; external hallib name 'libhal_psi_get_bool';

{ Get the value of a property of type string list.  }
function libhal_psi_get_strlist(iter: pLibHalPropertySetIterator): PPchar; cdecl; external hallib name 'libhal_psi_get_strlist';

{ Get the length of an array of strings  }
function libhal_string_array_length(str_array: Ppchar): dword; cdecl; external hallib name 'libhal_string_array_length';

{ Frees a NULL-terminated array of strings. If passed NULL, does nothing.  }
procedure libhal_free_string_array(str_array: Ppchar); cdecl; external hallib name 'libhal_free_string_array';

{ Frees a nul-terminated string  }
procedure libhal_free_string(str: pchar); cdecl; external hallib name 'libhal_free_string';

{ Create a new device object which will be hidden from applications
 * until the CommitToGdl(), ie. libhal_device_commit_to_gdl(), method is called.
  }
function libhal_new_device(ctx: pLibHalContext; error: pDBusError): Pchar; cdecl; external hallib name 'libhal_new_device';

{ When a hidden device has been built using the NewDevice method, ie.
 * libhal_new_device(), and the org.freedesktop.Hal.Device interface
 * this function will commit it to the global device list.}
function libhal_device_commit_to_gdl(ctx: pLibHalContext; temp_udi: pchar; udi: pchar; error: pDBusError): dbus_bool_t; cdecl; external hallib name 'libhal_device_commit_to_gdl';

{ This method can be invoked when a device is removed. The HAL daemon
 * will shut down the device. Note that the device may still be in the device
 * list if the Persistent property is set to true.
  }
function libhal_remove_device(ctx: pLibHalContext; udi: pchar; error: pDBusError): dbus_bool_t; cdecl; external hallib name 'libhal_remove_device';

{ Merge properties from one device to another.  }
function libhal_merge_properties(ctx: pLibHalContext; target_udi: pchar; source_udi: pchar; error: pDBusError): dbus_bool_t; cdecl; external hallib name 'libhal_merge_properties';

{ Check a set of properties for two devices matches.  }
function libhal_device_matches(ctx: pLibHalContext; udi1: pchar; udi2: pchar; property_namespace: pchar; error: pDBusError): dbus_bool_t; cdecl; external hallib name 'libhal_device_matches';

{ Find a device in the GDL where a single string property matches a
 * given value. }
function libhal_manager_find_device_string_match(ctx: pLibHalContext; key: pchar; value: pchar; num_devices: plongint; error: pDBusError): PPchar; cdecl; external hallib name 'libhal_manager_find_device_string_match';

{ Assign a capability to a device.  }
function libhal_device_add_capability(ctx: pLibHalContext; udi: pchar; capability: pchar; error: pDBusError): dbus_bool_t; cdecl; external hallib name 'libhal_device_add_capability';

{ Check if a device has a capability. The result is undefined if the
 * device doesn't exist. }
function libhal_device_query_capability(ctx: pLibHalContext; udi: pchar; capability: pchar; error: pDBusError): dbus_bool_t; cdecl; external hallib name 'libhal_device_query_capability';

{ Find devices with a given capability.  }
function libhal_find_device_by_capability(ctx: pLibHalContext; capability: pchar; num_devices: plongint; error: pDBusError): PPchar; cdecl; external hallib name 'libhal_find_device_by_capability';

{ Watch all devices, ie. the device_property_changed callback is
 * invoked when the properties on any device changes. }
function libhal_device_property_watch_all(ctx: pLibHalContext; error: pDBusError): dbus_bool_t; cdecl; external hallib name 'libhal_device_property_watch_all';

{ Add a watch on a device, so the device_property_changed callback is
 * invoked when the properties on the given device changes. }
function libhal_device_add_property_watch(ctx: pLibHalContext; udi: pchar; error: pDBusError): dbus_bool_t; cdecl; external hallib name 'libhal_device_add_property_watch';

{ Remove a watch on a device  }
function libhal_device_remove_property_watch(ctx: pLibHalContext; udi: pchar; error: pDBusError): dbus_bool_t; cdecl; external hallib name 'libhal_device_remove_property_watch';

{ Take an advisory lock on the device.  }
function libhal_device_lock(ctx: pLibHalContext; udi: pchar; reason_to_lock: pchar; reason_why_locked: Ppchar; error: pDBusError): dbus_bool_t; cdecl; external hallib name 'libhal_device_lock';

{ Release an advisory lock on the device.  }
function libhal_device_unlock(ctx: pLibHalContext; udi: pchar; error: pDBusError): dbus_bool_t; cdecl; external hallib name 'libhal_device_unlock';
function libhal_device_rescan(ctx: pLibHalContext; udi: pchar; error: pDBusError): dbus_bool_t; cdecl; external hallib name 'libhal_device_rescan';
function libhal_device_reprobe(ctx: pLibHalContext; udi: pchar; error: pDBusError): dbus_bool_t; cdecl; external hallib name 'libhal_device_reprobe';

{ Emit a condition from a device (for hald helpers only)  }
function libhal_device_emit_condition(ctx: pLibHalContext; udi: pchar; condition_name: pchar; condition_details: pchar; error: pDBusError): dbus_bool_t; cdecl; external hallib name 'libhal_device_emit_condition';

{ Claim an interface for a device (for hald helpers only)  }
function libhal_device_claim_interface(ctx: pLibHalContext; udi: pchar; interface_name: pchar; introspection_xml: pchar; error: pDBusError): dbus_bool_t; cdecl; external hallib name 'libhal_device_claim_interface';

{ hald waits for all addons to call this function before announcing the addon (for hald helpers only)  }
function libhal_device_addon_is_ready(ctx: pLibHalContext; udi: pchar; error: pDBusError): dbus_bool_t; cdecl; external hallib name 'libhal_device_addon_is_ready';

{ Take a mandatory lock on an interface on a device.  }
function libhal_device_acquire_interface_lock(ctx: pLibHalContext; udi: pchar; ainterface: pchar; exclusive: dbus_bool_t; error: pDBusError): dbus_bool_t; cdecl; external hallib name 'libhal_device_acquire_interface_lock';

{ Release a mandatory lock on an interface on a device.  }
function libhal_device_release_interface_lock(ctx: pLibHalContext; udi: pchar; ainterface: pchar; error: pDBusError): dbus_bool_t; cdecl; external hallib name 'libhal_device_release_interface_lock';

{ Take a mandatory lock on an interface (the lock affects all devices the caller have access to).  }
function libhal_acquire_global_interface_lock(ctx: pLibHalContext; ainterface: pchar; exclusive: dbus_bool_t; error: pDBusError): dbus_bool_t; cdecl; external hallib name 'libhal_acquire_global_interface_lock';

{ Release a mandatory lock on an interface (affects all devices the caller have access to).  }
function libhal_release_global_interface_lock(ctx: pLibHalContext; ainterface: pchar; error: pDBusError): dbus_bool_t; cdecl; external hallib name 'libhal_release_global_interface_lock';

{ Determine if a given caller is locked out of a given interface on a given device  }
function libhal_device_is_caller_locked_out(ctx: pLibHalContext; udi: pchar; ainterface: pchar; caller: pchar; error: pDBusError): dbus_bool_t; cdecl; external hallib name 'libhal_device_is_caller_locked_out';

{ Determines whether a determines other processes than the caller holds a lock on the given device.   }
function libhal_device_is_locked_by_others(ctx: pLibHalContext; udi: pchar; ainterface: pchar; error: pDBusError): dbus_bool_t; cdecl; external hallib name 'libhal_device_is_locked_by_others';


implementation


end.
