//Freepascal translation of lshal.c

(***************************************************************************
 * CVSID: $Id: lshal.c,v 1.21 2006-07-14 17:46:57 kay Exp $
 *
 * lshal.c : Show devices managed by HAL
 *
 * Copyright (C) 2003 David Zeuthen, <david@fubar.dk>
 * Copyright (C) 2005 Pierre Ossman, <drzeus@drzeus.cx>
 *
 * Licensed under the Academic Free License version 2.1
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *
 **************************************************************************)

program lshalpas;

{$MODE objfpc}{$H+}

uses
  cmem, Sysutils, glib2, dbus, libhal, getopts;

const UDI_BASE = '/org/freedesktop/Hal/devices/';

const long_options: array[0..8] of TOption = (
    (Name: 'monitor'; has_arg: no_argument; Flag: nil; Value: 'm'),
    (Name: 'long'; has_arg: no_argument; Flag: nil; Value: 'l'),
    (Name: 'short'; has_arg: no_argument; Flag: nil; Value: 's'),
    (Name: 'tree'; has_arg: no_argument; Flag: nil; Value: 't'),
    (Name: 'show'; has_arg: required_argument; Flag: nil; Value: 'u'),
    (Name: 'help'; has_arg: no_argument; Flag: nil; Value: 'h'),
    (Name: 'usage'; has_arg: no_argument; Flag: nil; Value: 'U'),
    (Name: 'version'; has_arg: no_argument; Flag: nil; Value: 'V'),
    (Name: ''; has_arg: no_argument; Flag: nil; Value: #0)
    );

var
  hal_ctx: PLibHalContext;
  long_list: boolean = false;
  tree_view: boolean = false;
  short_list: boolean = false;
  do_monitor: boolean = false;
  show_device: PChar = nil;
  loop: PGMainLoop;
  what: integer;
  c: char;
  error: DBusError;
  conn: PDBusConnection;
type

  pdevice = ^TDevice;
  Tdevice = record
    name: PChar;
    parent: PChar;
  end;

procedure dbus_connection_setup_with_g_main(connection: PDBusConnection; context: PGMainContext) cdecl; external 'dbus-glib-1' name 'dbus_connection_setup_with_g_main';

procedure printf(inp: PChar; args: array of const); overload;
begin
  write(Format(StringReplace(StringReplace(inp, '\n', LineEnding, [rfReplaceAll]),'"','''', [rfReplaceAll]), args));
end;

procedure printf(inp: PChar); overload;
begin
  write(StringReplace(StringReplace(inp, '\n', LineEnding, [rfReplaceAll]),'"','''', [rfReplaceAll]));
end;

procedure printflo(inp: PChar; args: array of const); overload;
begin
  write(lowercase(Format(StringReplace(StringReplace(inp, '\n', LineEnding, [rfReplaceAll]),'"','''', [rfReplaceAll]), args)));
end;

procedure fprintf(var F: Text; inp: PChar; args: array of const); overload;
begin
  write(F, Format(StringReplace(inp, '\n', LineEnding, [rfReplaceAll]), args));
end;

procedure fprintf(var F: Text; inp: PChar); overload;
begin
  write(F, StringReplace(inp, '\n', LineEnding, [rfReplaceAll]));
end;

function BoolToStrL(inp:Boolean):String;
begin
  Result:=LowerCase(BoolToStr(inp,true));
end;


function short_name(const udi: PChar): PChar;
begin
  result := @udi[sizeof(UDI_BASE)];
end;

procedure print_props(udi: PChar);
var error: DBusError;
  props: PLibHalPropertySet;
  it: LibHalPropertySetIterator;
  ptype: LibHalPropertyType;
  i: cardinal;
  strlist: PPChar;
begin
 dbus_error_init(@error);

  props := libhal_device_get_all_properties(hal_ctx, udi, @error);

  if (props = nil) then exit;

  libhal_psi_init(@it, props);

  repeat
    ptype := libhal_psi_get_type(@it);
    case ptype of
      LIBHAL_PROPERTY_TYPE_STRING:
        printf('  %s = "%s"  (string)\n', [libhal_psi_get_key(@it), libhal_psi_get_string(@it)]);
      LIBHAL_PROPERTY_TYPE_INT32:
        printflo('  %s = %d  (0x%x)  (int)\n', [
          libhal_psi_get_key(@it),
            libhal_psi_get_int(@it),
            libhal_psi_get_int(@it)]);
      LIBHAL_PROPERTY_TYPE_UINT64:
        printflo('  %s = %d  (0x%x)  (uint64)\n',
          [libhal_psi_get_key(@it),
          libhal_psi_get_uint64(@it),
            libhal_psi_get_uint64(@it)]);
      LIBHAL_PROPERTY_TYPE_DOUBLE:
        printf('  %s = %g  (double)\n',
          [libhal_psi_get_key(@it),
          libhal_psi_get_double(@it)]);
      LIBHAL_PROPERTY_TYPE_BOOLEAN:
        printf('  %s = %s  (bool)\n',
          [libhal_psi_get_key(@it),
          BoolToStrL(0 <> libhal_psi_get_bool(@it))]);
      LIBHAL_PROPERTY_TYPE_STRLIST:
        begin
          printf('  %s = {', [libhal_psi_get_key(@it)]);
          strlist := libhal_psi_get_strlist(@it);
          while strlist^ <> nil do
          begin
            printf('"%s"', [strlist^]);
            inc(strlist);
            if strlist^ <> nil then printf(', ');
          end;
          printf('} (string list)\n');
        end
    else printf('Unknown type %d=0x%02x\n', [ptype, ptype]);
    end;

    libhal_psi_next(@it);
  until libhal_psi_has_more(@it) = 0;

  libhal_free_property_set(props);
end;

procedure dump_device(udi: PChar);
var error: DBusError;
begin
  dbus_error_init(@error);

  if (0 = libhal_device_exists(hal_ctx, udi, @error)) then exit;

  if (long_list) then
  begin
    printf('udi = "%s"\n', [udi]);
    print_props(udi);
    printf('\n');
  end else
    printf('%s\n', [short_name(udi)]);
end;

procedure dump_children(udi: PChar; num_devices: integer; devices: pdevice; depth: integer);
var i, j: integer;
begin

  for i := 0 to num_devices - 1 do
  begin
    if (udi = nil) then
    begin
      if devices[i].parent <> nil then continue;
    end else
    begin
      if (devices[i].parent = nil) then continue;
      if (strcomp(devices[i].parent, udi) <> 0) then continue;
    end;

    if (long_list) then printf('udi = "%s"\n', [devices[i].name]) else
    begin
      if (tree_view) then
        for j := 0 to depth - 1 do printf('  ');
      printf('%s\n', [short_name(devices[i].name)]);
    end;

    if (long_list) then
    begin
      print_props(devices[i].name);
      printf('\n');
    end;

    dump_children(devices[i].name, num_devices, devices, depth + 1);
  end;

end;

procedure dump_devices;
var
  i, num_devices: integer;
  device_names: PPChar;
  devices: pdevice;
  error: DBusError;
begin

  dbus_error_init(@error);

  device_names := libhal_get_all_devices(hal_ctx, @num_devices, @error);
  if (device_names = nil) then
  begin
    printf(('Couldn''''t obtain list of devices\n'));
    exit;
  end;

  devices := malloc(sizeof(TDevice) * num_devices);
  if (devices = nil) then
  begin
    libhal_free_string_array(device_names);
    exit;
  end;

  for i := 0 to num_devices - 1 do begin
    devices[i].name := device_names[i];
    devices[i].parent := libhal_device_get_property_string(hal_ctx,
      device_names[i], 'info.parent', @error);

    if (dbus_error_is_set(@error) <> 0) then dbus_error_free(@error);
  end;

  if (long_list) then
    printf('\nDumping %d device(s) from the Global Device List:\n' +
      '-------------------------------------------------\n', [num_devices]);

  dump_children(nil, num_devices, devices, 0);

  for i := 0 to num_devices - 1 do
    if (devices[i].parent <> nil) then
      libhal_free_string(devices[i].parent);

  free(devices);
  libhal_free_string_array(device_names);

  if (long_list) then
    printf('\nDumped %d device(s) from the Global Device List.\n' +
      '------------------------------------------------\n\n',
      [num_devices]);
end;

procedure device_added(ctx: PLibHalContext; udi: PChar); cdecl;
begin

  if ((show_device <> nil) and (strcomp(show_device, udi) <> 0)) then exit;

  if (long_list) then
  begin
    printf('*** lshal: device_added, udi="%s"\n', [udi]);
    print_props(udi);
  end

  else printf('%s added\n', [short_name(udi)]);

end;

procedure device_removed(ctx: PLibHalContext; udi: PChar); cdecl;
begin
  if ((show_device <> nil) and (strcomp(show_device, udi) <> 0)) then exit;

  if (long_list) then
    printf('*** lshal: device_removed, udi="%s"\n', [udi])
  else printf('%s removed\n', [short_name(udi)]);

end;

procedure device_new_capability(ctx: PLibHalContext;
  udi: PChar;
  capability: PChar); cdecl;
begin
  if ((show_device <> nil) and (strcomp(show_device, udi) <> 0)) then exit;

  if (long_list) then
  begin
    printf('*** lshal: new_capability, udi="%s"\n', [udi]);
    printf('*** capability: %s\n', [capability]);
  end

  else printf('%s capability %s added\n', [short_name(udi), capability]);

end;

procedure device_lost_capability(ctx: PLibHalContext;
  udi: PChar;
  capability: PChar); cdecl;
begin
  if ((show_device <> nil) and (strcomp(show_device, udi) <> 0)) then exit;

  if (long_list) then
  begin
    printf('*** lshal: lost_capability, udi="%s"\n', [udi]);
    printf('*** capability: %s\n', [capability]);
  end

  else printf('%s capability %s lost\n', [short_name(udi), capability]);
end;

procedure print_property(const udi: PChar; const key: PChar);
var ptype: LibHalPropertyType;
  str: PChar;
  error: DBusError;
  value32: dbus_int32_t;
  value64: dbus_uint64_t;
  i: Cardinal;
  strlist, strlist_orig: PPChar;
  procedure printfl(inpa: PChar; inpb: PChar; args: array of const);
  begin
    if long_list then printflo(inpa, args) else printflo(inpb, args);
  end;
begin
  dbus_error_init(@error);
  ptype := libhal_device_get_property_type(hal_ctx, udi, key, @error);

  case ptype of
    LIBHAL_PROPERTY_TYPE_STRING:
      begin
        str := libhal_device_get_property_string(hal_ctx, udi, key, @error);
        printfl('*** new value: "%s"  (string)\n', '"%s"', [str]);
        libhal_free_string(str);
      end;
    LIBHAL_PROPERTY_TYPE_INT32:
      begin
        value32 := libhal_device_get_property_int(hal_ctx, udi, key, @error);
        printfl('*** new value: %d (0x%x)  (int)\n', '%d (0x%x)', [value32, value32]);
      end;
    LIBHAL_PROPERTY_TYPE_UINT64:
      begin
        value64 := libhal_device_get_property_uint64(hal_ctx, udi, key, @error);
        printfl('*** new value: %d (0x%x)  (uint64)\n', '%d (0x%x)', [value64, value64]);
      end;
    LIBHAL_PROPERTY_TYPE_DOUBLE:
      printfl('*** new value: %g  (double)\n', '%g',
        [libhal_device_get_property_double(hal_ctx, udi, key, @error)]);
    LIBHAL_PROPERTY_TYPE_BOOLEAN:
      printfl('*** new value: %s  (bool)\n', '%s',
        [BoolToStrL(0 <> libhal_device_get_property_bool(hal_ctx, udi, key, @error))]);
    LIBHAL_PROPERTY_TYPE_STRLIST:
      begin
        if (long_list) then printf('*** new value: {') else printf('{');
        strlist := libhal_device_get_property_strlist(hal_ctx, udi, key, @error);
        strlist_orig:=strlist;
        while strlist^ <> nil do
        begin
          printf('"%s"', [strlist^]);
          inc(strlist);
          if strlist^ <> nil then printf(', ');
        end;
        if (long_list) then printf('}  (string list)\n') else printf('}');
        libhal_free_string_array(strlist_orig);
      end;
  else fprintf(stderr, 'Unknown type %d="%c"\n', [ptype, ptype]);
  end;
end;

procedure property_modified(ctx: PLibHalContext;
  udi: PChar;
  key: PChar;
  is_removed: dbus_bool_t;
  is_added: dbus_bool_t); cdecl;
begin
  if ((show_device <> nil) and (strcomp(show_device, udi) <> 0)) then exit;

  if (long_list) then begin
    printf('*** lshal: property_modified, udi=%s, key=%s\n', [udi, key]);
    printf('           is_removed=%s, is_added=%s\n',
      [BoolToStrL(is_removed <> 0),
      BoolToStrL(is_added <> 0)]);
    if (is_removed = 0) then print_property(udi, key);
    printf('\n');
  end else
  begin
    printf('%s property %s ', [short_name(udi), key]);
    if (is_removed = 1) then printf('removed') else
    begin
      printf('= ');
      print_property(udi, key);
      if (is_added = 1) then printf(' (new)');
    end;
    printf('\n');
  end;

end;

procedure device_condition(ctx: PLibHalContext;
  udi: PChar;
  condition_name: PChar;
  condition_details: PChar); cdecl;
begin
  if ((show_device <> nil) and (strcomp(show_device, udi) <> 0)) then exit;

  if (long_list) then
  begin
    printf('*** lshal: device_condition, udi=%s\n', [udi]);
    printf('           condition_name=%s\n', [condition_name]);
    printf('           condition_details=%s\n', [condition_details]);
    printf('\n');
  end else printf('%s condition %s = %s\n', [short_name(udi),
      condition_name, condition_details]);

end;

procedure usage(argc: integer; argv: PPChar);
begin
  fprintf(stderr, '\nusage : %s [options]\n', [argv[0]]);
  fprintf(stderr,
    '\n' +
    'Options:\n' +
    '    -m, --monitor        Monitor device list\n' +
    '    -s, --short          short output (print only nonstatic part of udi)\n' +
    '    -l, --long           Long output\n' +
    '    -t, --tree           Tree view\n' +
    '    -u, --show <udi>     Show only the specified device\n' +
    '\n' +
    '    -h, --help           Show this information and exit\n' +
    '    -V, --version        Print version number\n' +
    '\n' +
    'Without any given options lshal will start with option --long.' +
    '\n' +
    'Shows all devices and their properties. If the --monitor option is given\n' +
    'then the device list and all devices are monitored for changes.\n' +
    '\n', []);
end;

begin
  while true do
  begin
    c := GetLongOpts('mlstu:hUV', long_options, what);
    if c = EndOfOptions then
    begin
      if ((do_monitor) and (long_list) and (short_list) and (tree_view) and (show_device = nil)) then
      begin
        usage(argc, argv);
        exit;
      end;
      break;
    end;

    case (c) of
      'm': do_monitor := TRUE;
      'l': long_list := TRUE;
      's': begin
          short_list := TRUE;
          long_list := FALSE;
        end;
      't': tree_view := TRUE;

      'u': begin
          if (pos('/', optarg) > 0) then show_device := PChar(AnsiString(optarg))
          else begin
            show_device := PChar(AnsiString(UDI_BASE + OptArg));
          end;
        end;

      'h', 'U': begin
          usage(argc, argv);
          exit;
        end;

      'V': begin
          printf('Freepascal libhal test\n');
          exit;
        end;

    else begin
        usage(argc, argv);
        exit;
      end;

    end;

  end;
  g_type_init;

  if (do_monitor) then loop := g_main_loop_new(nil, FALSE) else loop := NULL;

  dbus_error_init(@error);

  conn := dbus_bus_get(DBUS_BUS_SYSTEM, @error);

  if (conn = nil) then
  begin
    fprintf(stderr, 'error: dbus_bus_get: %s: %s\n',
      [error.name, error.message]);
    exit;
  end;

  if (do_monitor) then dbus_connection_setup_with_g_main(conn, nil);

  hal_ctx := libhal_ctx_new;

  if (hal_ctx = nil) then
  begin
    fprintf(stderr, 'error: libhal_ctx_new\n');
    exit;
  end;
  if (0 = libhal_ctx_set_dbus_connection(hal_ctx, conn)) then
  begin
    fprintf(stderr, 'error: libhal_ctx_set_dbus_connection: %s: %s\n',
      [error.name, error.message]);
    exit;
  end;
  if (0 = libhal_ctx_init(hal_ctx, @error)) then
  begin
    fprintf(stderr, 'error: libhal_ctx_init: %s: %s\n',
      [error.name, error.message]);
    fprintf(stderr, 'Could not initialise connection to hald. \n ' +
      'Normally this mean the HAL daemon (hald) is not running or not ready.\n');
    exit;
  end;

  libhal_ctx_set_device_added(hal_ctx, @device_added);
  libhal_ctx_set_device_removed(hal_ctx, @device_removed);
  libhal_ctx_set_device_new_capability(hal_ctx, @device_new_capability);
  libhal_ctx_set_device_lost_capability(hal_ctx, @device_lost_capability);
  libhal_ctx_set_device_property_modified(hal_ctx, @property_modified);
  libhal_ctx_set_device_condition(hal_ctx, @device_condition);

  if (show_device <> nil) then
    dump_device(show_device)
  else if (not do_monitor) then
    dump_devices();

  if (do_monitor and (loop <> nil)) then begin
    if (long_list or short_list or tree_view) then
      dump_devices();

    printf('\nStart monitoring devicelist:\n' +
      '-------------------------------------------------\n');
    libhal_device_property_watch_all(hal_ctx, @error);
    g_main_loop_run(loop);
  end;

  libhal_ctx_shutdown(hal_ctx, @error);
  libhal_ctx_free(hal_ctx);

 // dbus_connection_close (conn); //->error
  dbus_connection_unref(conn);

end.
